@@ -22,6 +22,12 @@ from . import web
 
 devmode = "--dev" in sys.argv
 
+DEFAULT_SCRIPT = "main.py"
+CACHE_ROOT = Path("build")
+CACHE_PATH = CACHE_ROOT / "web-cache"
+CACHE_VERSION = CACHE_ROOT / "version.txt"
+CACHE_APP = CACHE_ROOT / "web"
+
 cdn_dot = __version__.split(".")
 cdn_dot.pop()
 cdn_version = ".".join(cdn_dot)
@@ -50,56 +56,114 @@ else:
     DEFAULT_PORT = 8000
     DEFAULT_TMPL = "default.tmpl"
 
-DEFAULT_SCRIPT = "main.py"
 
+def set_args(program):
+    global DEFAULT_SCRIPT
+    import sys
+    from pathlib import Path
 
-def main():
-    asyncio.run(main_run(Path(sys.argv[-1]).resolve()))
-
-
-async def main_run(patharg, cdn=DEFAULT_CDN):
-    global DEFAULT_PORT, DEFAULT_SCRIPT
+    required = []
 
+    patharg = Path(program).resolve()
     if patharg.is_file():
-        DEFAULT_SCRIPT = patharg.name
+        mainscript = patharg.name
         app_folder = patharg.parent
     else:
         app_folder = patharg.resolve()
+        mainscript = DEFAULT_SCRIPT
 
-    reqs = []
+    sys.path.insert(0, str(app_folder))
 
-    if sys.version_info < (3, 8):
-        # zip deflate  compression level 3.7
-        # https://docs.python.org/3.11/library/shutil.html#shutil.copytree dirs_exist_ok = 3.8
-        reqs.append("pygbag requires CPython version >= 3.8")
+    if patharg.suffix == "pyw":
+        required.append("79: Error, no .pyw allowed use .py for python script")
+
+    script_path = app_folder / mainscript
+
+    if not script_path.is_file():
+        required.append(f"83: Error, no main.py {script_path} found in folder")
 
     if not app_folder.is_dir() or patharg.as_posix().endswith("/pygbag/__main__.py"):
-        reqs.append(
-            "ERROR: Last argument must be app top level directory, or the main python script"
+        required.append(
+            "89: Error, Last argument must be a valid app top level directory, or the main.py python script"
         )
 
-    if len(reqs):
-        while len(reqs):
-            print(reqs.pop())
-        sys.exit(1)
+    if sys.version_info < (3, 8):
+        # zip deflate compression level 3.7
+        # https://docs.python.org/3.11/library/shutil.html#shutil.copytree dirs_exist_ok = 3.8
+        required.append("84: Error, pygbag requires CPython version >= 3.8")
 
-    app_folder.joinpath("build").mkdir(exist_ok=True)
+    if len(required):
+        while len(required):
+            print(required.pop())
+        print("89: missing requirement(s)")
+        sys.exit(89)
 
-    build_dir = app_folder.joinpath("build/web")
-    build_dir.mkdir(exist_ok=True)
+    return app_folder, mainscript
 
-    cache_dir = app_folder.joinpath("build/web-cache")
 
-    if devmode and cache_dir.is_dir():
-        print("DEVMODE: clearing cache")
+def cache_check(app_folder, devmode=False):
+    global CACHE_PATH, CACHE_APP, __version__
+
+    version_file = app_folder / CACHE_VERSION
+
+    clear_cache = False
+
+    # always clear the cache in devmode, because cache source is local and changes a lot
+    if devmode:
+        print("103: DEVMODE: clearing cache")
+        clear_cache = True
+    elif version_file.is_file():
+        try:
+            with open(version_file, "r") as file:
+                cache_ver = file.read()
+                if cache_ver != __version__:
+                    print(
+                        f"115: cache {cache_ver} mismatch, want {__version__}, cleaning ..."
+                    )
+                    clear_cache = True
+        except:
+            # something's wrong in cache structure, try clean it up
+            clear_cache = True
+    else:
+        clear_cache = True
+
+    cache_root = app_folder.joinpath(CACHE_ROOT)
+    cache_dir = app_folder / CACHE_PATH
+    build_dir = app_folder / CACHE_APP
+
+    def make_cache_dirs():
+        nonlocal cache_root, cache_dir
+
+        cache_root.mkdir(exist_ok=True)
+        build_dir.mkdir(exist_ok=True)
+        cache_dir.mkdir(exist_ok=True)
+
+    if clear_cache:
         if shutil.rmtree.avoids_symlink_attacks:
-            shutil.rmtree(cache_dir.as_posix())
+            if cache_dir.is_dir():
+                shutil.rmtree(cache_dir.as_posix())
         else:
-            print("can't clear cache : rmtree is not safe")
+            print(
+                "115: cannot clear cache : rmtree is not safe on that system",
+                file=sys.stderr,
+            )
+            raise SystemExit(115)
 
-    cache_dir.mkdir(exist_ok=True)
+        # rebuild
+        make_cache_dirs()
 
-    # version = "0.0.0"
+        with open(version_file, "w") as file:
+            file.write(__version__)
+
+    return build_dir, cache_dir
+
+
+async def main_run(app_folder, mainscript, cdn=DEFAULT_CDN):
+    global DEFAULT_PORT, DEFAULT_SCRIPT, APP_CACHE, required
+
+    DEFAULT_SCRIPT = mainscript or DEFAULT_SCRIPT
+
+    build_dir, cache_dir = cache_check(app_folder, devmode)
 
     sys.argv.pop()
 
@@ -236,7 +300,7 @@ ________________________
 # the app folder
 app_folder={app_folder}
 
-# artefacts directoty
+# artefacts directory
 build_dir={build_dir}
 
 # the window title and icon name
@@ -245,7 +309,7 @@ app_name={app_name}
 # package name, better make it unique
 package={args.package}
 
-# icon 96x96 for dekstop 16x16 for web
+# icons:  96x96 for desktop, 16x16 for web
 icon={args.icon}
 
 # js/wasm provider
@@ -411,3 +475,17 @@ build_dir = {build_dir}
             )
     else:
         print(args.template, "is not a valid template")
+
+
+def main():
+    app_folder, mainscript = set_args(sys.argv[-1])
+
+    # sim does not use cache.
+    if "--sim" in sys.argv:
+        print(
+            f"To use simulator launch with : {sys.executable} -m pygbag {' '.join(sys.argv[1:])}"
+        )
+        return 1
+    else:
+        asyncio.run(main_run(app_folder, mainscript))
+    return 0
